package listener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import model.Play;
import scratch.ScratchFile;
import scratch.ScratchPanel;
import scratch.ScratchProgram;
import theater_intern.TheaterObservable;
import util.IO;
import util.PropertyManager;
import util.ResourceManager;
import editor.Editor;
import editor.SolistEditor;

public class NewProgramListener implements ActionListener, Observer {

	JMenuItem item;
	JButton button;

	public NewProgramListener(JMenuItem item, JButton button) {
		this.item = item;
		this.button = button;
		TheaterObservable.getObservable().addObserver(this);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		createNewProgram();
	}

	public void createNewProgram() {
		try {
			Play play = Play.getPlay();
			if (!(play.isSimulator() && PropertyManager.getPropertyManager()
					.isScratch())) {
				Editor editor = SolistEditor.getSolistEditor().getEditor();
				if (!editor.isSavedAs()) {
					int answer = JOptionPane.showConfirmDialog(Play.getPlay()
							.getPlayFrame(), ResourceManager
							.getResourceManager().getValue("msg.savequestion1")
							+ "\n"
							+ ResourceManager.getResourceManager().getValue(
									"msg.savequestion"), ResourceManager
							.getResourceManager().getValue("msg.request"),
							JOptionPane.YES_NO_CANCEL_OPTION);
					if (answer == JOptionPane.YES_OPTION) {
						if (!SaveProgramListener.saveProgram()) {
							return;
						}
					} else if (answer == JOptionPane.CANCEL_OPTION) {
						return;
					}
				}
				if (Play.getPlay().isImperativeParadigma()) {
					IO.copyResTextFile(SolistEditor.CLASS_NAME + ".txt", play
							.getDirectory()
							+ File.separatorChar
							+ SolistEditor.CLASS_NAME
							+ ".txt");
				} else {
					String classname = Play.getPlay().getActiveActorName();
					if (classname == null) {
						classname = "theater.Actor";
					}
					IO.copyResTextFile("OO" + SolistEditor.CLASS_NAME + ".txt",
							play.getDirectory() + File.separatorChar
									+ SolistEditor.CLASS_NAME + ".txt",
							classname);
				}
				File to = new File(play.getDirectory() + File.separatorChar
						+ SolistEditor.CLASS_NAME + ".txt");
				editor.getEditorPanel().loadFile(to);
				editor.setChanged(true);
				Play.getPlay().getPlayFrame().getMessagePanel().writeInfo(
						ResourceManager.getResourceManager().getValue(
								"msg.program.new"));
			} else {
				try {
					ScratchPanel scratchPanel = Play.getPlay().getPlayFrame()
							.getScratchPanel();
					if (scratchPanel.isModified()) {
						int answer = JOptionPane.showConfirmDialog(Play
								.getPlay().getPlayFrame(), ResourceManager
								.getResourceManager().getValue(
										"msg.savequestion1")
								+ "\n"
								+ ResourceManager.getResourceManager()
										.getValue("msg.savequestion"),
								ResourceManager.getResourceManager().getValue(
										"msg.request"),
								JOptionPane.YES_NO_CANCEL_OPTION);
						if (answer == JOptionPane.YES_OPTION) {
							if (!SaveProgramListener.saveProgram()) {
								return;
							}
						} else if (answer == JOptionPane.CANCEL_OPTION) {
							return;
						}
					}
					ScratchProgram p = ScratchFile.newProgram();
					scratchPanel.setProgram(p);
					Play.getPlay().getPlayFrame().getMessagePanel().writeInfo(
							ResourceManager.getResourceManager().getValue(
									"msg.program.new"));
				} catch (Exception exc) {
					JOptionPane.showMessageDialog(
							Play.getPlay().getPlayFrame(), ResourceManager
									.getResourceManager().getValue(
											"msg.editor.newerror"),
							ResourceManager.getResourceManager().getValue(
									"msg.error"), JOptionPane.ERROR_MESSAGE);
				}
			}
		} catch (Throwable th) {
			th.printStackTrace();
		}

	}

	@Override
	public void update(Observable o, Object arg) {
		if (!Play.getPlay().getActivePerformance().simulationStopped()) {
			item.setEnabled(false);
			if (button != null) {
				button.setEnabled(false);
			}
		} else {
			item.setEnabled(true);
			if (button != null) {
				button.setEnabled(true);
			}
		}

	}

}
